package de.ordix.onews.persistence.dao;

import java.util.List;

import javax.persistence.EntityManager;

import de.ordix.onews.persistence.util.NoResultDaoException;


/**
* Licensed to the Apache Software Foundation (ASF) under one or more
* contributor license agreements. See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* The ASF licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
* 
 * Eine Dao zur Erstellung einer Entity mit den nötigen CRUD-Operationen 
 * 
 * @author sk@ordix.de
 *
 * @param <T>: Typ der Entity
 * @param <P>: Typ des Primary Keys
 */
public interface Dao<T, P> {
    T save(T object);

    T getById(P id) throws NoResultDaoException;

    T update(T object);

    void delete(T object);

    public List<T> selectAll() throws NoResultDaoException;

    void setEntityManager(EntityManager entityManager);

    /**
     * Es werden alle Objekte zurückgegeben, die dem Muster im Example-Objekt entsprechen.
     * 
     * @param example
     *            das Objekt hat dieselben Attribute, wie das Ziel-Objekt.
     *            Allerdings sind die Attribute des Example-Objekts alle vom Typ
     *            String mit Ausnahme von Enums
     * @return
     * @throws NoResultDaoException 
     */
    public List<T> findByExample(Object example) throws NoResultDaoException;
    
 
   
    

    
}
