package de.ordix.news.test.util;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
* contributor license agreements. See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* The ASF licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
* 
 * Hilfsklasse für die Persistenzschicht zum Erstellen einer
 * EntityManagerFactory und eines EntityManager . Vor der ersten Verwendung muss
 * das Attribut persistenceUnit bestimmt werden!
 * 
 * Die Klasse dient bloß für die JUnit-Tests!
 * 
 * ToDo: Muss in den Test-Paket verschoben werden
 * 
 * @author sk
 *
 */
public class EntityManagerUtil {

    private static EntityManagerFactory entityManagerFactory;
    private static EntityManager entityManager = null;

    public synchronized static EntityManagerFactory getEntityManagerFactory() {
        if (entityManagerFactory == null) {
            createEntityManagerFactory();
        }
        return entityManagerFactory;
    }

    private static void createEntityManagerFactory() {
        entityManagerFactory = Persistence.createEntityManagerFactory("skh2");
    }

    public static EntityManager getEntityManager() {
        if (entityManager == null) {
            entityManager = getEntityManagerFactory().createEntityManager();
        }
        return entityManager;
    }

    public static EntityTransaction getTransaction() {
        return getEntityManager().getTransaction();
    }

    /**
     * Soll eine andere Konfiguration verwendet werden, muessen die bestehende
     * Ressourcen freigegeben werden.
     */
    public static void close() {
        if (entityManager != null && entityManager.isOpen()) {
            entityManager.close();
        }
        if (entityManagerFactory != null && entityManagerFactory.isOpen()) {
            entityManagerFactory.close();
        }
        entityManager = null;
        entityManagerFactory = null;
    }
}
