package de.ordix.onews.persistence.impl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.time.LocalDate;
import java.util.List;

import javax.persistence.EntityManager;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import de.ordix.news.test.util.EntityManagerUtil;
import de.ordix.onews.persistence.entity.Aenderungskennzeichen;
import de.ordix.onews.persistence.entity.Personal;
import de.ordix.onews.persistence.entity.PersonalExample;
import de.ordix.onews.persistence.entity.Upload;
import de.ordix.onews.persistence.entity.UploadExample;
import de.ordix.onews.persistence.util.NoResultDaoException;

/*
* Licensed to the Apache Software Foundation (ASF) under one or more
* contributor license agreements. See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* The ASF licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

public class PersonalDaoImplTest {

	private PersonalDaoImpl personalDao = null;
	private EntityManager entityManager;

	@Before
	public void setUp() throws Exception {
		entityManager = EntityManagerUtil.getEntityManager();
		personalDao = new PersonalDaoImpl();
		personalDao.setEntityManager(entityManager);
	}

	@After
	public void tearDown() throws Exception {
		EntityManagerUtil.close();
	}

	@Test
	public void findPersonalAnhandUpload() throws NoResultDaoException {
		Personal personal = createPersonalWithUpload("Koch", "Stefan", "meineDatei");
		personal = save(personal);

		UploadExample upExample = new UploadExample();
		upExample.setDateiname("meine%");
		PersonalExample pExample = new PersonalExample();
		pExample.setUpload(upExample);
		List<Personal> ergebnisListe = findByExample(pExample);
		assertEquals(1, ergebnisListe.size());
	}

	@Test(expected=NoResultDaoException.class)
	public void dontFindPersonalAnhandUpload() throws NoResultDaoException {
		Personal personal = createPersonalWithUpload("Koch", "Stefan", "meineDatei");
		personal = save(personal);

		UploadExample upExample = new UploadExample();
		upExample.setDateiname("falscher Dateiname");
		PersonalExample pExample = new PersonalExample();
		pExample.setUpload(upExample);
		findByExample(pExample);
		fail("Exception wird erwartet");
	}

	private List<Personal> findByExample(PersonalExample personalExample) throws NoResultDaoException {
		EntityManagerUtil.getTransaction().begin();
		List<Personal> returnValue = personalDao.findByExample(personalExample);
		EntityManagerUtil.getTransaction().commit();
		return returnValue;
	}

	private Personal save(Personal personal) {
		EntityManagerUtil.getTransaction().begin();
		Personal returnValue = personalDao.save(personal);
		EntityManagerUtil.getTransaction().commit();
		return returnValue;
	}

	private Personal createPersonalWithUpload(String name, String vorname, String dateiname) {
		Personal returnValue = createPersonal(name, vorname);
		Upload upload = createUpload(dateiname);
		returnValue.setUpload(upload);
		return returnValue;
	}
	private Personal createPersonal(String name, String vorname) {
		Personal returnValue = new Personal();
		returnValue.setNachname(name);
		returnValue.setVorname(vorname);
		returnValue.setGeburtsdatum(LocalDate.now().minusYears(42));
		returnValue.setAenderungskennzeichen(Aenderungskennzeichen.ZUGANG);
		return returnValue;
	}

	private Upload createUpload(String dateiname) {
		Upload returnValue = new Upload();
		returnValue.setDateiname(dateiname);
		return returnValue;
	}
}
